'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _slug = require('slug');

var _slug2 = _interopRequireDefault(_slug);

var _slash = require('slash');

var _slash2 = _interopRequireDefault(_slash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Makeup = function () {
  function Makeup(generator) {
    (0, _classCallCheck3.default)(this, Makeup);

    this.generator = generator;
    this.config = {};
  }

  (0, _createClass3.default)(Makeup, [{
    key: 'hasFeature',
    value: function hasFeature(feature) {
      return _lodash2.default.isArray(this.config.features) && this.config.features.indexOf(feature) >= 0;
    }
  }, {
    key: 'setAppname',
    value: function setAppname(appname) {
      if (_lodash2.default.isString(appname)) {
        this.config.appname = appname;
      }
    }
  }, {
    key: 'setDirs',
    value: function setDirs(dirs) {
      if (_lodash2.default.isObject(dirs)) {
        this.config.dirs = dirs;
      } else {
        this.config.dirs = {};
      }
    }
  }, {
    key: 'setUrls',
    value: function setUrls(urls) {
      if (_lodash2.default.isObject(urls)) {
        this.config.urls = urls;
      } else {
        this.config.urls = {};
      }
    }
  }, {
    key: 'setFramework',
    value: function setFramework(framework) {
      if (Makeup.allFrameworks.indexOf(framework) >= 0) {
        this.config.framework = framework;
      }
    }
  }, {
    key: 'setFrontend',
    value: function setFrontend(frontend) {
      if (Makeup.allFrontends.indexOf(frontend) >= 0) {
        this.config.frontend = frontend;
      }
    }
  }, {
    key: 'setFeatures',
    value: function setFeatures(features) {
      if (_lodash2.default.isArray(features)) {
        this.config.features = (0, _lodash2.default)(Makeup.allFeatures).filter(function (feature) {
          return features.indexOf(feature) >= 0;
        }).value();
      }
    }
  }, {
    key: 'addFeatures',
    value: function addFeatures(features) {
      var _this = this;

      if (!_lodash2.default.isArray(this.config.features)) this.config.features = [];
      if (!_lodash2.default.isArray(features)) features = [features];
      this.config.features = (0, _lodash2.default)(Makeup.allFeatures).filter(function (feature) {
        return _this.config.features.indexOf(feature) >= 0 || features.indexOf(feature) >= 0;
      }).value();
    }
  }, {
    key: 'removeFeatures',
    value: function removeFeatures(features) {
      var _this2 = this;

      if (!_lodash2.default.isArray(this.config.features)) this.config.features = [];
      if (!_lodash2.default.isArray(features)) features = [features];
      this.config.features = (0, _lodash2.default)(Makeup.allFeatures).filter(function (feature) {
        return _this2.config.features.indexOf(feature) >= 0 && !features.indexOf(feature) >= 0;
      }).value();
    }
  }, {
    key: 'applyConfig',
    value: function applyConfig(config) {
      // console.log('applyConfig config=', config);
      this.setAppname(config.appname);
      this.setDirs(config.dirs);
      this.setUrls(config.urls);
      this.setFramework(config.framework);
      this.setFrontend(config.frontend);
      this.setFeatures(config.features);
    }
  }, {
    key: 'applyOptions',
    value: function applyOptions(options) {
      // console.log('applyOptions options=', options);
      this.setAppname(options.appname);
      _lodash2.default.merge(this.config.dirs, {
        bower: options.bowerDir,
        src: options.srcDir,
        test: options.testDir,
        tmp: options.tmpDir,
        tmpDev: options.tmpDevDir,
        tmpTest: options.tmpTestDir,
        dist: options.distDir
      });
      _lodash2.default.merge(this.config.urls, {
        staticBase: options.staticBaseUrl
      });
    }
  }, {
    key: 'applyDefaults',
    value: function applyDefaults(appname) {
      var config = this.config;

      var _$defaults = _lodash2.default.defaults(config, {
        appname: appname,
        dirs: {},
        urls: {}
      }),
          dirs = _$defaults.dirs,
          urls = _$defaults.urls;

      _lodash2.default.defaults(dirs, {
        bower: 'bower_components',
        src: 'src',
        test: 'test',
        tmp: '.tmp',
        dist: 'dist'
      });
      _lodash2.default.defaults(dirs, {
        tmpDev: (0, _slash2.default)(_path2.default.join(dirs.tmp, 'dev')),
        tmpTest: (0, _slash2.default)(_path2.default.join(dirs.tmp, 'test'))
      });

      _lodash2.default.defaults(urls, {
        staticBase: '/__static__'
      });

      _lodash2.default.defaults(config, {
        framework: 'marionette',
        frontend: 'foundation'
      });

      if (!config.features) this.setFeatures(['modernizr', 'sass', 'crusher']);
    }
  }, {
    key: 'getQuestions',
    value: function getQuestions() {
      var _this3 = this;

      var config = this.config;

      return [{
        type: 'list',
        name: 'framework',
        message: 'What model-view-* framework would you like to use?',
        choices: [{
          name: 'None',
          value: 'none'
        }, {
          name: 'Backbone',
          value: 'backbone'
        }, {
          name: 'Backbone + Marionette',
          value: 'marionette'
        }],
        default: config.framework
      }, {
        type: 'list',
        name: 'frontend',
        message: 'What Front-end framework would you like to use?',
        choices: [{
          name: 'None',
          value: 'none'
        }, {
          name: 'Foundation',
          value: 'foundation'
        }, {
          name: 'Bootstrap',
          value: 'bootstrap'
        }],
        default: config.frontend
      }, {
        type: 'checkbox',
        name: 'features',
        message: 'What extra features would you like?',
        choices: function choices(answers) {
          var choices = [{
            name: 'Server side rendering',
            value: 'serverRender',
            checked: _this3.hasFeature('serverRender'),
            disabled: answers.framework === 'none' ? 'required if no framework selected' : false
          }, {
            name: 'Modernizr',
            value: 'modernizr',
            checked: _this3.hasFeature('modernizr'),
            disabled: answers.frontend === 'foundation' ? 'required by foundation' : false
          }, {
            name: 'Sass',
            value: 'sass',
            checked: _this3.hasFeature('sass'),
            disabled: answers.frontend === 'foundation' ? 'required by foundation' : answers.frontend === 'bootstrap' ? 'required by bootstrap' : false
          }, {
            name: 'Cache-crusher',
            value: 'crusher',
            checked: _this3.hasFeature('crusher')
          }];
          return choices;
        }
      }];
    }
  }, {
    key: 'autoAdjust',
    value: function autoAdjust() {
      var config = this.config;

      if (config.framework === 'none') this.addFeatures(['serverRender']);
      if (config.frontend === 'foundation') this.addFeatures(['modernizr', 'sass']);
      if (config.frontend === 'bootstrap') this.addFeatures(['sass']);
    }
  }, {
    key: 'putAnswers',
    value: function putAnswers(answers) {
      this.setFramework(answers.framework);
      this.setFrontend(answers.frontend);
      this.setFeatures(answers.features);
      if (!answers.noAdjust) this.autoAdjust();
    }
  }, {
    key: 'getTemplateConfig',
    value: function getTemplateConfig() {
      var config = this.config;

      return {
        appname: config.appname,
        appnameSlug: (0, _slug2.default)(config.appname),
        appnameCap: _lodash2.default.capitalize(config.appname),
        dirs: config.dirs,
        urls: config.urls,
        use: {
          backbone: config.framework === 'backbone' || config.framework === 'marionette',
          marionette: config.framework === 'marionette',
          foundation: config.frontend === 'foundation',
          bootstrap: config.frontend === 'bootstrap',
          serverRender: this.hasFeature('serverRender'),
          modernizr: this.hasFeature('modernizr'),
          sass: this.hasFeature('sass'),
          crusher: this.hasFeature('crusher')
        }
      };
    }
  }, {
    key: 'getPermanetConfig',
    value: function getPermanetConfig() {
      return this.config;
    }
  }]);
  return Makeup;
}();

Makeup.allFrameworks = ['none', 'backbone', 'marionette'];
Makeup.allFrontends = ['none', 'foundation', 'bootstrap'];
Makeup.allFeatures = ['serverRender', 'modernizr', 'sass', 'crusher'];

exports.default = Makeup;